/* -------------------------------------------------------------------------- *
 * file:	zengingen.c                                                   *
 * purpose:	functions for zengin file content access and manipulation     *
 *              used by showzengin, buildzengin and decodezengin              *
 * ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <cgic.h>
#include "zengingen.h"

static int hdr_field_length[HDR_FIELDS]     = {1, 2, 1, 10, 40, 4, 4, 15, 3, 15, 1, 7, 3, 14};
static int data_field_length[DATA_FIELDS]   = {1, 4, 15, 3, 15, 4, 1, 7, 30, 10, 1, 10, 10, 1, 1, 7};
static int tlr_field_length[TRAILER_FIELDS] = {1, 6, 12, 101};
static int end_field_length[END_FIELDS]     = {1, 119};

char *hdr_fieldnames[HDR_FIELDS] = { "Record Type", "Transaction Type", "Character Encoding", "Requestor ID", "Account Holder Name", "Payment Value Date", "Remitting Bank Number", "Remitting Bank Name", "Remitting Branch Number", "Remitting Branch Name", "Requestor Account Type", "Requestor Account Number", "FC Payment Currency", "Fill Spaces" };

char *hdr_fielddescr[HDR_FIELDS] = {
	"1 indicates this is a Header Record.",
	"Automated Clearing House (ACH) Electronic Funds Transfer Transaction Code (21,22 = Checking Account Credit, 32 = Savings Account Credit).",
	"Character encoding parameter: 0=JIS, 1=EBCDIC.",
	"10 digits debit account number, right aligned with leading 0.",
	"40 characters customer name in single-byte Katakana.",
	"4-digit payment value date (format: DDMM).",
	"4-digit Zengin bank code (example: 0401=Citibank).",
	"15 characters remitting bank name in single-byte Katakana, left aligned and filled up with spaces.",
	"3-digit Zengin bank branch code (example Citibank: 730=Tokyo Branch)",
	"15 characters bank branch name in single-byte Katakana, left aligned and filled up with spaces.",
	"Account type 1 = Ordinary,  2 = Checking, 9 = Unknown.",
	"7-digit beginning of the sender bank account number, right aligned.",
	"In case of foreign currency payments, this field carries the 3-digit ISO currency code. Domestic payments in JPY need 3 spaces here.",
	"14 blank (space) characters fill the record up to its required length of 120 bytes." };

char *data_fieldnames[DATA_FIELDS] = { "Record Type", "Receiving Bank Number", "Receiving Bank Name", "Receiving Branch Number", "Receiving Branch Name", "Clearinghouse Number", "Account Type", "Account Number", "Account Owner Name", "Payment Amount", "New / Amend", "Customer Reference 1", "Customer Reference 2", "Transfer Priority", "Identification Display", "Fill Spaces" };
char *data_fielddescr[DATA_FIELDS] = {
        "2 indicates this is a Data Record.",
        "4-digit Zengin code of the beneficiary bank.",
        "15 characters name of the beneficiary bank.",
        "3-digit Zengin code of the beneficiary bank branch office.",
        "15 characters name of the beneficiary bank branch office.",
        "4-digit Zengin code of the clearing house.",
        "1-digit beneficiary account type (1 = Ordinary, 2 = Checking, 4 = Unknown, 9 = Unknown).",
        "7-digit beneficiary account number, right aligned with leading zero.",
        "40 characters left-aligned single-byte Katakana name, space fills blanks",
        "10 digits aligned right with leading zeros. Foreign currencies lower denomination units are in the mostright 2-digits.",
        "1 character New value (space) or correction.",
        "10 characters customer reference field 1.",
        "10 characters customer reference field 2.",
        "1-digit transfer priority (7 = Express, 8 = Normal)",
        "1 character, choices are Y and space.",
        "7 blank (space) characters fill the record up to its required length of 120 bytes." };

char *tlr_fieldnames[TRAILER_FIELDS] = { "Record Type", "Data Records", "Total Amount", "Fill Spaces" };
char *tlr_fielddescr[TRAILER_FIELDS] = { "9 indicates this is a Trailer Record.",
	"6-digit number of data records in this file.",
	"12 digits are the total amount of payments in this file (summary).",
	"101 blank (space) characters fill the record up to its required length of 120 bytes." };

char *end_fieldnames[END_FIELDS] = { "Record Type", "Fill Spaces" };
char *end_fielddescr[END_FIELDS] = { "9 indicates this is a End Record.", "119 blank (space) characters fill the record up to its required length of 120 bytes." };

/* -------------------------------------------------------------------------- *
 * zengin_read() - read the zengin file content into the records structure    *
 * ---------------------------------------------------------------------------*/
void zengin_read(char * zenginbuf) {
  int filepos    = 0;
  int recordpos  = 0;
  int recordcnt  = 0;

  /* -------------- *
   * Header Record  *
   * ---------------*/
  if(zenginbuf[filepos] != '1') int_error("Zengin Format Error: Expecting Header Record Marker.");
  for(recordpos=0;recordpos<RECLEN;recordpos++) {
    if(zenginbuf[filepos] == '\0') { int_error("Zengin Format Error: Unexpected End in Header Record."); break; }
    zengin_records.hdr_record[recordpos] = zenginbuf[filepos];
    filepos++;
  }
  zengin_records.hdr_record[RECLEN]='\0';

  /* --------------------------------------------------------------------------* 
   * Format Check. CitiDirect specifies a CR\LF (hex 0d 0a) after each record, * 
   * we check if the next byte is a data record marker, and if not if it is a  * 
   * CR. Then we check if a LF follows, and if yes voila the file is from Citi * 
   * --------------------------------------------------------------------------*/
  zengin_fmt = 0;
  if ( zengin_fmt == 0 && zenginbuf[filepos] == '2') zengin_fmt = NO_CRLF_FMT;
  if ( zengin_fmt == 0 && zenginbuf[filepos] == '\n') { zengin_fmt = LF_FMT; filepos++; }
  else {
    if (zengin_fmt == 0 && zenginbuf[filepos] == '\r') {
      filepos++;
      if ( zengin_fmt == 0 && zenginbuf[filepos] == '\n') { zengin_fmt = CRLF_FMT; filepos++; }
      else { int_error("Zengin Format Error: Received CR but no LF, garbled Citi file?"); }
    } else { int_error("Zengin Format Error: Expecting data record start marker '2', LF or CR."); }
  }

  /* ------------- *
   * Data Record   *
   * --------------*/
  for(recordcnt=0;recordcnt<MAXDATA;recordcnt++) {
    if(zenginbuf[filepos] != '2') break;
    recordpos  = 0;
    for(recordpos=0;recordpos<RECLEN;recordpos++) {
      if(zenginbuf[filepos] == '\0') { int_error("Zengin Format Error: Unexpected End in Data Record."); break; }
      zengin_records.data_records[recordcnt][recordpos] = zenginbuf[filepos];
      filepos++;
    }
    zengin_records.data_records[recordcnt][RECLEN]='\0';
    if (zengin_fmt == CRLF_FMT) { filepos++; filepos++; }
    if (zengin_fmt == LF_FMT) { filepos++; }
  }

  if(recordcnt == 0) int_error("Zengin Format Error: Could not find Data Record.");
  else data_records = recordcnt;

  /* --------------- *
   * Trailer Record  *
   * ----------------*/
  recordpos  = 0;
  if(zenginbuf[filepos] != '8') int_error("Zengin Format Error: Expecting Trailer Record Marker.");
  for(recordpos=0;recordpos<RECLEN;recordpos++) {
    if(zenginbuf[filepos] == '\0') { int_error("Zengin Format Error: Unexpected End in Trailer Record."); break; }
    zengin_records.tlr_record[recordpos] = zenginbuf[filepos];
    filepos++;
  }
  zengin_records.tlr_record[RECLEN]='\0';
  if (zengin_fmt == CRLF_FMT) { filepos++; filepos++; }
  if (zengin_fmt == LF_FMT) { filepos++; }

  /* ------------- *
   * End - Record  *
   * --------------*/
  recordpos  = 0;
  if(zenginbuf[filepos] != '9') int_error("Zengin Format Error: Expecting Trailer Record Marker.");
  for(recordpos=0;recordpos<RECLEN;recordpos++) {
    if(zenginbuf[filepos] == '\0') { int_error("Zengin Format Error: Unexpected End in Trailer Record."); break; }
    zengin_records.end_record[recordpos] = zenginbuf[filepos];
    filepos++;
  }
  zengin_records.end_record[RECLEN]='\0';
  if (zengin_fmt == CRLF_FMT) { filepos++; filepos++; }
  if (zengin_fmt == LF_FMT) { filepos++; }
}
/* -------------------------------------------------------------------------- *
 * get_valuedate() - get 4-digit MMDD payment value date from Header Record   *
 * ---------------------------------------------------------------------------*/
void get_valuedate() {
  int i;

  if(strlen(zengin_records.hdr_record) == 120) {
    for(i=0; i<4; i++) valuedate[i] = zengin_records.hdr_record[54+i];
    valuedate[4] = '\0';
    if(valuedate[0] == ' ') int_error("Zengin Format Error: No valid Date in Header Record.");
  } else {
    int_error("Zengin Format Error: Header Record Data incomplete or n/a.");
  }
}
/* -------------------------------------------------------------------------- *
 * get_currency() - get 3-digit ISO currency code from Header Record          *
 * ---------------------------------------------------------------------------*/
void get_currency() {
  int i;

  if(strlen(zengin_records.hdr_record) == 120) {
    for(i=0; i<3; i++) currency[i] = zengin_records.hdr_record[103+i];
    currency[3] = '\0';
    if(currency[0] == ' ') strcpy(currency, "YEN");
  } else {
    int_error("Zengin Format Error: Header Record Data incomplete or n/a.");
  }
}
/* -------------------------------------------------------------------------- *
 * get_total() - get total amount from Trailer Record                         *
 * ---------------------------------------------------------------------------*/
void get_total() {
  char total_amountstr[13] = "";
  int i;

  if(strlen(zengin_records.tlr_record) == 120) {
    for(i=0; i<12; i++) total_amountstr[i] = zengin_records.tlr_record[7+i];
    total_amountstr[12] = '\0';
    total_amount = atoi(total_amountstr);
  } else {
    int_error("Zengin Format Error: Trailer Record Data incomplete or n/a.");
  }
}
/* -------------------------------------------------------------------------- *
 * write_rawtable() - write the zengin records structure into a HTML table    *
 * mode 0 = display, mode 1 = edit (form input)                               *
 * ---------------------------------------------------------------------------*/
void write_rawtable(int mode) {
  int i         = 0;
  int j         = 0;
  int k         = 0;
  int recordcnt = 0;
  int bytecount = 0;
  div_t oddline_calc;

  fprintf(cgiOut, "<table>");

 /* ---------------------------------------------------------------------------*
  * Header Record Display                                                             *
  * ---------------------------------------------------------------------------*/
  fprintf(cgiOut, "<tr>");
  fprintf(cgiOut, "<th colspan=\"31\">");
  fprintf(cgiOut, "Header");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");
  j=0; k=0;
  for(i=0;i<30;i++) {
    if(k == hdr_field_length[j]) { j++; k=0; }
    oddline_calc = div(j, 2);
    if(oddline_calc.rem) {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"hdrin_odd\">");
        fprintf(cgiOut, "<input class=\"hdr_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.hdr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"hdrout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, hdr_fieldnames[j], hdr_fielddescr[j]);
        if (zengin_records.hdr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.hdr_record[i]);
      }
    } else {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"hdrin_even\">");
        fprintf(cgiOut, "<input class=\"hdr_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.hdr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"hdrout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, hdr_fieldnames[j], hdr_fielddescr[j]);
        if (zengin_records.hdr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.hdr_record[i]);
      }
    }
    fprintf(cgiOut, "</td>\n");
    k++; bytecount++;
  }
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "001-030");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  for(i=30;i<60;i++) {
    if(k == hdr_field_length[j]) { j++; k=0; }
    oddline_calc = div(j, 2);
    if(oddline_calc.rem) {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"hdrin_odd\">");
        fprintf(cgiOut, "<input class=\"hdr_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.hdr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"hdrout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, hdr_fieldnames[j], hdr_fielddescr[j]);
        if (zengin_records.hdr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.hdr_record[i]);
      }
    } else {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"hdrin_even\">");
        fprintf(cgiOut, "<input class=\"hdr_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.hdr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"hdrout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, hdr_fieldnames[j], hdr_fielddescr[j]);
        if (zengin_records.hdr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.hdr_record[i]);
      }
    }
    fprintf(cgiOut, "</td>\n");
    k++; bytecount++;
  }
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "031-060");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  for(i=60;i<90;i++) {
    if(k == hdr_field_length[j]) { j++; k=0; }
    oddline_calc = div(j, 2);
    if(oddline_calc.rem) {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"hdrin_odd\">");
        fprintf(cgiOut, "<input class=\"hdr_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.hdr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"hdrout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, hdr_fieldnames[j], hdr_fielddescr[j]);
        if (zengin_records.hdr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.hdr_record[i]);
      }
    } else {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"hdrin_even\">");
        fprintf(cgiOut, "<input class=\"hdr_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.hdr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"hdrout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, hdr_fieldnames[j], hdr_fielddescr[j]);
        if (zengin_records.hdr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.hdr_record[i]);
      }
    }
    fprintf(cgiOut, "</td>\n");
    k++; bytecount++;
  }
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "061-090");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  for(i=90;i<120;i++) {
    if(k == hdr_field_length[j]) { j++; k=0; }
    oddline_calc = div(j, 2);
    if(oddline_calc.rem) {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"hdrin_odd\">");
        fprintf(cgiOut, "<input class=\"hdr_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.hdr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"hdrout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, hdr_fieldnames[j], hdr_fielddescr[j]);
        if (zengin_records.hdr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.hdr_record[i]);
      }
    } else {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"hdrin_even\">");
        fprintf(cgiOut, "<input class=\"hdr_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.hdr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"hdrout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, hdr_fieldnames[j], hdr_fielddescr[j]);
        if (zengin_records.hdr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.hdr_record[i]);
      }
    }
    fprintf(cgiOut, "</td>\n");
    k++; bytecount++;
  }
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "091-120");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");
 /* ---------------------------------------------------------------------------*
  * Data Record Display                                                        *
  * ---------------------------------------------------------------------------*/
  while(zengin_records.data_records[recordcnt][0] == '2') {
    fprintf(cgiOut, "<tr>");
    fprintf(cgiOut, "<th colspan=\"31\">");
    fprintf(cgiOut, "Data %02d", recordcnt+1);
    fprintf(cgiOut, "</th>\n");
    fprintf(cgiOut, "</tr>\n");
    j=0; k=0;
    for(i=0;i<30;i++) {
      if(k == data_field_length[j]) { j++; k=0; }
      oddline_calc = div(j, 2);
      if(oddline_calc.rem) {
        if(mode == 1) {
          fprintf(cgiOut, "<td class=\"datain_odd\">");
          fprintf(cgiOut, "<input class=\"data_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.data_records[recordcnt][i]);
        } else {
          fprintf(cgiOut, "<td class=\"dataout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, data_fieldnames[j], data_fielddescr[j]);
          if (zengin_records.data_records[recordcnt][i] == ' ') fprintf(cgiOut, "&nbsp;");
          else fprintf(cgiOut, "%c", zengin_records.data_records[recordcnt][i]);
        }
      } else {
        if(mode == 1) {
          fprintf(cgiOut, "<td class=\"datain_even\">");
          fprintf(cgiOut, "<input class=\"data_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.data_records[recordcnt][i]);
        } else {
          fprintf(cgiOut, "<td class=\"dataout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, data_fieldnames[j], data_fielddescr[j]);
          if (zengin_records.data_records[recordcnt][i] == ' ') fprintf(cgiOut, "&nbsp;");
          else fprintf(cgiOut, "%c", zengin_records.data_records[recordcnt][i]);
        }
      }
      fprintf(cgiOut, "</td>\n");
      k++; bytecount++;
    }
    fprintf(cgiOut, "<th>");
    fprintf(cgiOut, "001-030");
    fprintf(cgiOut, "</th>\n");
    fprintf(cgiOut, "</tr>\n");
  
    fprintf(cgiOut, "<tr>\n");
    for(i=30;i<60;i++) {
      if(k == data_field_length[j]) { j++; k=0; }
      oddline_calc = div(j, 2);
      if(oddline_calc.rem) {
        if(mode == 1) {
          fprintf(cgiOut, "<td class=\"datain_odd\">");
          fprintf(cgiOut, "<input class=\"data_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.data_records[recordcnt][i]);
        } else {
          fprintf(cgiOut, "<td class=\"dataout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, data_fieldnames[j], data_fielddescr[j]);
          if (zengin_records.data_records[recordcnt][i] == ' ') fprintf(cgiOut, "&nbsp;");
          else fprintf(cgiOut, "%c", zengin_records.data_records[recordcnt][i]);
        }
      } else {
        if(mode == 1) {
          fprintf(cgiOut, "<td class=\"datain_even\">");
          fprintf(cgiOut, "<input class=\"data_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.data_records[recordcnt][i]);
        } else {
          fprintf(cgiOut, "<td class=\"dataout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, data_fieldnames[j], data_fielddescr[j]);
          if (zengin_records.data_records[recordcnt][i] == ' ') fprintf(cgiOut, "&nbsp;");
          else fprintf(cgiOut, "%c", zengin_records.data_records[recordcnt][i]);
        }
      }
      fprintf(cgiOut, "</td>\n");
      k++; bytecount++;
    }
    fprintf(cgiOut, "<th>");
    fprintf(cgiOut, "031-060");
    fprintf(cgiOut, "</th>\n");
    fprintf(cgiOut, "</tr>\n");

    fprintf(cgiOut, "<tr>\n");
    for(i=60;i<90;i++) {
      if(k == data_field_length[j]) { j++; k=0; }
      oddline_calc = div(j, 2);
      if(oddline_calc.rem) {
        if(mode == 1) {
          fprintf(cgiOut, "<td class=\"datain_odd\">");
          fprintf(cgiOut, "<input class=\"data_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.data_records[recordcnt][i]);
        } else {
          fprintf(cgiOut, "<td class=\"dataout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, data_fieldnames[j], data_fielddescr[j]);
          if (zengin_records.data_records[recordcnt][i] == ' ') fprintf(cgiOut, "&nbsp;");
          else fprintf(cgiOut, "%c", zengin_records.data_records[recordcnt][i]);
        }
      } else {
        if(mode == 1) {
          fprintf(cgiOut, "<td class=\"datain_even\">");
          fprintf(cgiOut, "<input class=\"data_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.data_records[recordcnt][i]);
        } else {
          fprintf(cgiOut, "<td class=\"dataout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, data_fieldnames[j], data_fielddescr[j]);
          if (zengin_records.data_records[recordcnt][i] == ' ') fprintf(cgiOut, "&nbsp;");
          else fprintf(cgiOut, "%c", zengin_records.data_records[recordcnt][i]);
        }
      }
      fprintf(cgiOut, "</td>\n");
      k++; bytecount++;
    }
    fprintf(cgiOut, "<th>");
    fprintf(cgiOut, "061-090");
    fprintf(cgiOut, "</th>\n");
    fprintf(cgiOut, "</tr>\n");

  
    fprintf(cgiOut, "<tr>\n");
    for(i=90;i<120;i++) {
      if(k == data_field_length[j]) { j++; k=0; }
      oddline_calc = div(j, 2);
      if(oddline_calc.rem) {
        if(mode == 1) {
          fprintf(cgiOut, "<td class=\"datain_odd\">");
          fprintf(cgiOut, "<input class=\"data_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.data_records[recordcnt][i]);
        } else {
          fprintf(cgiOut, "<td class=\"dataout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, data_fieldnames[j], data_fielddescr[j]);
          if (zengin_records.data_records[recordcnt][i] == ' ') fprintf(cgiOut, "&nbsp;");
          else fprintf(cgiOut, "%c", zengin_records.data_records[recordcnt][i]);
        }
      } else {
        if(mode == 1) {
          fprintf(cgiOut, "<td class=\"datain_even\">");
          fprintf(cgiOut, "<input class=\"data_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.data_records[recordcnt][i]);
        } else {
          fprintf(cgiOut, "<td class=\"dataout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, data_fieldnames[j], data_fielddescr[j]);
          if (zengin_records.data_records[recordcnt][i] == ' ') fprintf(cgiOut, "&nbsp;");
          else fprintf(cgiOut, "%c", zengin_records.data_records[recordcnt][i]);
        }
      }
      fprintf(cgiOut, "</td>\n");
      k++; bytecount++;
    }
    fprintf(cgiOut, "<th>");
    fprintf(cgiOut, "091-120");
    fprintf(cgiOut, "</th>\n");
    fprintf(cgiOut, "</tr>\n");

    recordcnt++;
  }
 /* ---------------------------------------------------------------------------*
  * Trailer Record Display                                                            *
  * ---------------------------------------------------------------------------*/
  fprintf(cgiOut, "<tr>");
  fprintf(cgiOut, "<th colspan=\"31\">");
  fprintf(cgiOut, "Trailer");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");
  j=0; k=0;
  for(i=0;i<30;i++) {
    if(k == tlr_field_length[j]) { j++; k=0; }
    oddline_calc = div(j, 2);
    if(oddline_calc.rem) {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"tlrin_odd\">");
        fprintf(cgiOut, "<input class=\"tlr_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.tlr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"tlrout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, tlr_fieldnames[j], tlr_fielddescr[j]);
        if (zengin_records.tlr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.tlr_record[i]);
      }
    } else {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"tlrin_even\">");
        fprintf(cgiOut, "<input class=\"tlr_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.tlr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"tlrout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, tlr_fieldnames[j], tlr_fielddescr[j]);
        if (zengin_records.tlr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.tlr_record[i]);
      }
    }
    fprintf(cgiOut, "</td>\n");
    k++; bytecount++;
  }
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "001-030");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  for(i=30;i<60;i++) {
    if(k == tlr_field_length[j]) { j++; k=0; }
    oddline_calc = div(j, 2);
    if(oddline_calc.rem) {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"tlrin_odd\">");
        fprintf(cgiOut, "<input class=\"tlr_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.tlr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"tlrout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, tlr_fieldnames[j], tlr_fielddescr[j]);
        if (zengin_records.tlr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.tlr_record[i]);
      }
    } else {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"tlrin_even\">");
        fprintf(cgiOut, "<input class=\"tlr_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.tlr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"tlrout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, tlr_fieldnames[j], tlr_fielddescr[j]);
        if (zengin_records.tlr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.tlr_record[i]);
      }
    }
    fprintf(cgiOut, "</td>\n");
    k++; bytecount++;
  }
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "031-060");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  for(i=60;i<90;i++) {
    if(k == tlr_field_length[j]) { j++; k=0; }
    oddline_calc = div(j, 2);
    if(oddline_calc.rem) {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"tlrin_odd\">");
        fprintf(cgiOut, "<input class=\"tlr_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.tlr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"tlrout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, tlr_fieldnames[j], tlr_fielddescr[j]);
        if (zengin_records.tlr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.tlr_record[i]);
      }
    } else {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"tlrin_even\">");
        fprintf(cgiOut, "<input class=\"tlr_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.tlr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"tlrout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, tlr_fieldnames[j], tlr_fielddescr[j]);
        if (zengin_records.tlr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.tlr_record[i]);
      }
    }
    fprintf(cgiOut, "</td>\n");
    k++; bytecount++;
  }
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "061-090");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");


  fprintf(cgiOut, "<tr>\n");
  for(i=90;i<120;i++) {
    if(k == tlr_field_length[j]) { j++; k=0; }
    oddline_calc = div(j, 2);
    if(oddline_calc.rem) {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"tlrin_odd\">");
        fprintf(cgiOut, "<input class=\"tlr_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.tlr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"tlrout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, tlr_fieldnames[j], tlr_fielddescr[j]);
        if (zengin_records.tlr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.tlr_record[i]);
      }
    } else {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"tlrin_even\">");
        fprintf(cgiOut, "<input class=\"tlr_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.tlr_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"tlrout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, tlr_fieldnames[j], tlr_fielddescr[j]);
        if (zengin_records.tlr_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.tlr_record[i]);
      }
    }
    fprintf(cgiOut, "</td>\n");
    k++; bytecount++;
  }
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "091-120");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");
 /* ---------------------------------------------------------------------------*
  * End Record Display, Field color alternates between BB9999 and FF9999       *
  * ---------------------------------------------------------------------------*/
  fprintf(cgiOut, "<tr>");
  fprintf(cgiOut, "<th colspan=\"31\">");
  fprintf(cgiOut, "End");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");
  j=0; k=0;
  for(i=0;i<30;i++) {
    if(k == end_field_length[j]) { j++; k=0; }
    oddline_calc = div(j, 2);
    if(oddline_calc.rem) {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"endin_odd\">");
        fprintf(cgiOut, "<input class=\"end_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.end_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"endout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, end_fieldnames[j], end_fielddescr[j]);
        if (zengin_records.end_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.end_record[i]);
      }
    } else {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"endin_even\">");
        fprintf(cgiOut, "<input class=\"end_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.end_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"endout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, end_fieldnames[j], end_fielddescr[j]);
        if (zengin_records.end_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.end_record[i]);
      }
    }
    fprintf(cgiOut, "</td>\n");
    k++; bytecount++;
  }
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "001-030");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  for(i=30;i<60;i++) {
    if(k == end_field_length[j]) { j++; k=0; }
    oddline_calc = div(j, 2);
    if(oddline_calc.rem) {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"endin_odd\">");
        fprintf(cgiOut, "<input class=\"end_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.end_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"endout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, end_fieldnames[j], end_fielddescr[j]);
        if (zengin_records.end_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.end_record[i]);
      }
    } else {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"endin_even\">");
        fprintf(cgiOut, "<input class=\"end_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.end_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"endout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, end_fieldnames[j], end_fielddescr[j]);
        if (zengin_records.end_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.end_record[i]);
      }
    }
    fprintf(cgiOut, "</td>\n");
    k++; bytecount++;
  }
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "031-060");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  for(i=60;i<90;i++) {
    if(k == end_field_length[j]) { j++; k=0; }
    oddline_calc = div(j, 2);
    if(oddline_calc.rem) {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"endin_odd\">");
        fprintf(cgiOut, "<input class=\"end_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.end_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"endout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, end_fieldnames[j], end_fielddescr[j]);
        if (zengin_records.end_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.end_record[i]);
      }
    } else {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"endin_even\">");
        fprintf(cgiOut, "<input class=\"end_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.end_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"endout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, end_fieldnames[j], end_fielddescr[j]);
        if (zengin_records.end_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.end_record[i]);
      }
    }
    fprintf(cgiOut, "</td>\n");
    k++; bytecount++;
  }
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "061-090");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");

  fprintf(cgiOut, "<tr>\n");
  for(i=90;i<120;i++) {
    if(k == end_field_length[j]) { j++; k=0; }
    oddline_calc = div(j, 2);
    if(oddline_calc.rem) {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"endin_odd\">");
        fprintf(cgiOut, "<input class=\"end_odd\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.end_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"endout_odd\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, end_fieldnames[j], end_fielddescr[j]);
        if (zengin_records.end_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.end_record[i]);
      }
    } else {
      if(mode == 1) {
        fprintf(cgiOut, "<td class=\"endin_even\">");
        fprintf(cgiOut, "<input class=\"end_even\" type=\"text\" name=\"zen_c%d\" size=1 maxlength=1 value=\"%c\">", bytecount, zengin_records.end_record[i]);
      } else {
        fprintf(cgiOut, "<td class=\"endout_even\" onClick=\"alert('Field %d: %s - Description: %s')\">", j+1, end_fieldnames[j], end_fielddescr[j]);
        if (zengin_records.end_record[i] == ' ') fprintf(cgiOut, "&nbsp;");
        else fprintf(cgiOut, "%c", zengin_records.end_record[i]);
      }
    }
    fprintf(cgiOut, "</td>\n");
    k++; bytecount++;
  }
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "091-120");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</tr>\n");
 /* ---------------------------------------------------------------------------*/
  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<th colspan=\"32\">");
  fprintf(cgiOut, "Zengin File - %d Bytes ", bytecount);
  if (zengin_fmt == CRLF_FMT) fprintf(cgiOut, "with CRLF Termination");
  if (zengin_fmt == LF_FMT) fprintf(cgiOut, "with LF Termination");
  if (zengin_fmt == NO_CRLF_FMT) fprintf(cgiOut, "without CRLF Termination");
  if (zengin_fmt == 0) fprintf(cgiOut, "Format: unknown");
  fprintf(cgiOut, "</th>");
  fprintf(cgiOut, "</tr>\n");
  fprintf(cgiOut, "</table>\n");
}

/* -------------------------------------------------------------------------- *
 * add_crlf() adds the CRLF bytes to a zengin_filestr                         *
 * ---------------------------------------------------------------------------*/
void add_crlf() {
  int i = 0;
  int j = 0;
  int k = 0;

  while(zengin_filestr[i]) {
    if(j == RECLEN) {
      zengin_filebuf[k] = '\r';
      k++;
      zengin_filebuf[k] = '\n';
      k++; j=0;
    } else {
      zengin_filebuf[k] = zengin_filestr[i];
      i++; j++; k++;
    }
  }
  strcpy(zengin_filestr, zengin_filebuf);
}
