/* -------------------------------------------------------------------------- *
 * file:         zenginstore.c                                                *
 * purpose:      display the list of signed certificates                      *
 * -------------------------------------------------------------------------- */
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <cgic.h>
#include "zengingen.h"

//int hexsort(const void *test1, const void *test2) {
int hexsort(const struct dirent  **test1, const struct dirent **test2) {
  char *endptr;
  return (strtol((*test1)->d_name, &endptr, 16)
        - strtol((*test2)->d_name, &endptr, 16));
}

int file_select(const struct dirent *entry) {

  /* check for "." and ".." directory entries */
  if(entry->d_name[0]=='.') return 0;

  /* Check for zengin-<id> file name extensions */
  if(strstr(entry->d_name, "zengin-") != NULL) return 1;
  else return 0;
}

int cgiMain() {

  static char      title[]           = "List of existing Zengin Files";
         char      sorting[16]       = "desc";
         char      zfileurl[225]     = "";
  struct dirent    **zstore_files    = NULL;
         int       pagenumber        = 1;
         int       zengincounter     = 0;
         int       tempcounter       = 0;
         int       pagecounter       = 0;
         int       dispcounter       = 0;
         int       dispmaxlines      = 0;
         div_t     disp_calc;
         div_t     oddline_calc;
         int       i;
/* -------------------------------------------------------------------------- *
 * Get the list of zengin files from the store directory                      *
 * ---------------------------------------------------------------------------*/
  zengincounter = scandir(ZENGINSTORE, &zstore_files, file_select, hexsort);
  if(zengincounter<=0) int_error("Error: No zenging files found.");

/* -------------------------------------------------------------------------- *
 * calculate how many pages we get with MAXFILEDISPLAY                         *
 * ---------------------------------------------------------------------------*/

  if(zengincounter<=MAXFILEDISPLAY) pagecounter = 1;
  else {
    disp_calc = div(zengincounter, MAXFILEDISPLAY);
    /* if the count of certs divided by MAXFILEDISPLAY has no remainder */
    if(disp_calc.rem == 0) pagecounter = disp_calc.quot;
    /* with a remainder, we must prepare an extra page for the rest */
    else pagecounter = disp_calc.quot +1;
  }

/* -------------------------------------------------------------------------- *
 * Check if we have been subsequently called with a pagenumber & sort request *
 * ---------------------------------------------------------------------------*/

  if(cgiFormInteger("page", &pagenumber, 1) == cgiFormSuccess)
    if(pagenumber > pagecounter || pagenumber <=0)
      int_error("Error: Page does not exist.");

  if(cgiFormString("sort", sorting, sizeof(sorting)) != cgiFormSuccess)
      strncpy(sorting, "desc", sizeof(sorting));

/* -------------------------------------------------------------------------- *
 * now we know how many certs we have in total and we can build the page(s).  *
 * For every MAXFILEDISPLAY certs we start a new page and cycle through by    *
 * calling ourself with the requested certs in range.                         *
 * ---------------------------------------------------------------------------*/

  if(strcmp(sorting, "asc") == 0) {

    if(zengincounter <= MAXFILEDISPLAY) {
       dispmaxlines = zengincounter;
       tempcounter = 0;
    }
    else
      if(pagenumber == pagecounter &&
             ( pagecounter * MAXFILEDISPLAY) - zengincounter != 0) {

        tempcounter = (pagecounter * MAXFILEDISPLAY) - MAXFILEDISPLAY;
        dispmaxlines = zengincounter - ((pagecounter-1) * MAXFILEDISPLAY);
      }
      else {

        tempcounter = (pagenumber * MAXFILEDISPLAY) - MAXFILEDISPLAY;
        dispmaxlines = MAXFILEDISPLAY;
      }
  }

  if(strcmp(sorting, "desc") == 0) {

    if(zengincounter <= MAXFILEDISPLAY) {
       dispmaxlines = zengincounter;
       tempcounter = zengincounter;
    }
    else
      if(pagenumber == pagecounter &&
             ( pagecounter * MAXFILEDISPLAY) - zengincounter != 0) {

        tempcounter = zengincounter - ((pagecounter-1) * MAXFILEDISPLAY);
        dispmaxlines = zengincounter - ((pagecounter-1) * MAXFILEDISPLAY);
      }
      else {

       tempcounter = zengincounter - (pagenumber*MAXFILEDISPLAY) + MAXFILEDISPLAY;
       dispmaxlines = MAXFILEDISPLAY;
      }
  }

/* -------------------------------------------------------------------------- *
 * start the html output                                                      *
 * ---------------------------------------------------------------------------*/

  pagehead(title);

  //debugging only:
  //printf("Number of files: %d\n", zengincounter);
  //printf("Num tempcounter: %d\n", tempcounter);
  //printf("Number of pages: %d\n", pagecounter);
  //printf("Div Quotient: %d\n", disp_calc.quot);
  //printf("Div Remainder: %d\n", disp_calc.rem);
  //fprintf(cgiOut, "</BODY></HTML>\n");
  //exit(0);

/* -------------------------------------------------------------------------- *
 * start the form output                                                      *
 * ---------------------------------------------------------------------------*/

   fprintf(cgiOut, "<table>");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<th width=\"20\">");
   fprintf(cgiOut, "#");
   fprintf(cgiOut, "</th>");
   fprintf(cgiOut, "<th width=\"240\">");
   fprintf(cgiOut, "Zengin File Name");
   fprintf(cgiOut, "</th>");
   fprintf(cgiOut, "<th width=\"70\">");
   fprintf(cgiOut, "Payments");
   fprintf(cgiOut, "</th>");
   fprintf(cgiOut, "<th width=\"120\">");
   fprintf(cgiOut, "AmountTotal");
   fprintf(cgiOut, "</th>");
   fprintf(cgiOut, "<th width=\"70\">");
   fprintf(cgiOut, "ValueDate");
   fprintf(cgiOut, "</th>");
   fprintf(cgiOut, "<th width=\"100\">");
   fprintf(cgiOut, "Action");
   fprintf(cgiOut, "</th>");
   fprintf(cgiOut, "</tr>\n");

  for(dispcounter=0; dispcounter < dispmaxlines; dispcounter++) {

    if(strcmp(sorting, "desc") == 0) tempcounter--;

    snprintf(zfileurl, sizeof(zfileurl), "../store/%s",
                           zstore_files[tempcounter]->d_name);

  /* -------------------------------------------------------------------------- *
   * Try to open, read and parse the zengin file                                *
   * ---------------------------------------------------------------------------*/
    snprintf(zfilepath, sizeof(zfilepath), "%s/%s", ZENGINSTORE, zstore_files[tempcounter]->d_name);
    if (! (zenginfile = fopen(zfilepath, "r")))
      int_error("Error cant open zengin file");
    for(i=0;i<sizeof(zengin_filestr);i++) zengin_filestr[i] = getc(zenginfile);
    fclose(zenginfile);
    /* parse content */
    zengin_read(zengin_filestr);
    get_valuedate();
    get_currency();
    get_total();

    fprintf(cgiOut, "<tr>");
    fprintf(cgiOut, "<th>");
    fprintf(cgiOut, "%d", tempcounter+1);
    fprintf(cgiOut, "</th>\n");

    oddline_calc = div(tempcounter+1, 2);
    if(oddline_calc.rem)
      fprintf(cgiOut, "<td bgcolor=\"FFFFFF\">");
    else
      fprintf(cgiOut, "<td bgcolor=\"CFCFCF\">");

    fprintf(cgiOut, "<font size=\"2\" face=\"Arial\">");
    fprintf(cgiOut, "<a href=\"%s\">%s</a>",
                     zfileurl, zstore_files[tempcounter]->d_name);
    fprintf(cgiOut, "</font></td>");

    /* payments count column */
    if(oddline_calc.rem)
      fprintf(cgiOut, "<td bgcolor=\"FFFFFF\" align=right>");
    else
      fprintf(cgiOut, "<td bgcolor=\"CFCFCF\" align=right>");
    fprintf(cgiOut, "%d", data_records);
    fprintf(cgiOut, "</td>");

    /* total amount column */
    if(oddline_calc.rem)
      fprintf(cgiOut, "<td bgcolor=\"FFFFFF\" align=right>");
    else
      fprintf(cgiOut, "<td bgcolor=\"CFCFCF\" align=right>");
    if(strcmp(currency, "YEN") == 0) fprintf(cgiOut, "%d %s", total_amount, currency);
    else fprintf(cgiOut, "%'.2f %s", (double) total_amount/100, currency);
    fprintf(cgiOut, "</td>");

    /* value date column */
    if(oddline_calc.rem)
      fprintf(cgiOut, "<td bgcolor=\"FFFFFF\" align=right>");
    else
      fprintf(cgiOut, "<td bgcolor=\"CFCFCF\" align=right>");
    fprintf(cgiOut, "%c%c/%c%c", valuedate[0],valuedate[1],valuedate[2],valuedate[3]);
    fprintf(cgiOut, "</td>");

    /* action column */
    fprintf(cgiOut, "<form action=\"showzengin.cgi\" method=\"post\">");
    fprintf(cgiOut, "<th>");
    fprintf(cgiOut, "<input type=\"hidden\" name=\"zfilename\" ");
    fprintf(cgiOut, "value=%s>", zstore_files[tempcounter]->d_name);
    fprintf(cgiOut, "<input type=\"hidden\" name=\"format\" value=\"pem\">");
    fprintf(cgiOut, "<input type=\"submit\" style=\"width:95px\" value=\"ViewRAW\">");
    fprintf(cgiOut, "</th>");
    fprintf(cgiOut, "</form>");
    fprintf(cgiOut, "</tr>");

    if(strcmp(sorting, "asc") == 0) tempcounter++;
  }

  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<th colspan=\"6\">");
  fprintf(cgiOut, "Total # of files: %d | ", zengincounter);
  fprintf(cgiOut, "Page %d of %d", pagenumber, pagecounter);
  fprintf(cgiOut, "</th>");
  fprintf(cgiOut, "</tr>");
  fprintf(cgiOut, "</table>\n");

  fprintf(cgiOut, "<p>");
  fprintf(cgiOut, "<table>");
  fprintf(cgiOut, "<tr>\n");
  fprintf(cgiOut, "<form action=\"zenginstore.cgi\" method=\"post\">");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"sort\" ");
  fprintf(cgiOut, "value=\"desc\">\n");
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "<input type=\"submit\" name=\"sort\"");
  fprintf(cgiOut, " value=\"Latest File First\">");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</form>");

  fprintf(cgiOut, "<form action=\"zenginstore.cgi\" method=\"post\">");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"sort\" ");
  fprintf(cgiOut, "value=\"asc\">\n");
  fprintf(cgiOut, "<th>");
  fprintf(cgiOut, "<input type=\"submit\" name=\"sort\"");
  fprintf(cgiOut, " value=\"Oldest File First\">");
  fprintf(cgiOut, "</th>\n");
  fprintf(cgiOut, "</form>");

  // filler 1
  fprintf(cgiOut, "<th width=55>");
  fprintf(cgiOut, "&nbsp;");
  fprintf(cgiOut, "</th>\n");

  // goto page 1
  fprintf(cgiOut, "<form action=\"zenginstore.cgi\" method=\"post\">");
  fprintf(cgiOut, "<th width=5>");
  fprintf(cgiOut, "<input type=\"submit\" value=\"<<\">");
  fprintf(cgiOut, "</th>");
  fprintf(cgiOut, "</form>");
  // goto page before
  fprintf(cgiOut, "<form action=\"zenginstore.cgi\" method=\"post\">");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"zengincounter\" ");
  fprintf(cgiOut, "value=\"");
  fprintf(cgiOut, "%d", zengincounter);
  fprintf(cgiOut, "\">\n");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"pagecounter\" ");
  fprintf(cgiOut, "value=\"");
  fprintf(cgiOut, "%d", pagecounter);
  fprintf(cgiOut, "\">\n");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"page\" ");
  fprintf(cgiOut, "value=\"");
  tempcounter = 0;
  if(pagenumber > 1) tempcounter = pagenumber - 1;
  else tempcounter = 1;
  fprintf(cgiOut, "%d", tempcounter);
  fprintf(cgiOut, "\">\n");
  fprintf(cgiOut, "<th width=5>");
  fprintf(cgiOut, "<input type=\"submit\" value=\"< 1\">");
  fprintf(cgiOut, "</th>");
  fprintf(cgiOut, "</form>");

  // filler 2
  fprintf(cgiOut, "<th width=10>");
  fprintf(cgiOut, "&nbsp;");
  fprintf(cgiOut, "</th>\n");

  // goto page after
  fprintf(cgiOut, "<form action=\"zenginstore.cgi\" method=\"post\">");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"zengincounter\" ");
  fprintf(cgiOut, "value=\"");
  fprintf(cgiOut, "%d", zengincounter);
  fprintf(cgiOut, "\">\n");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"pagecounter\" ");
  fprintf(cgiOut, "value=\"");
  fprintf(cgiOut, "%d", pagecounter);
  fprintf(cgiOut, "\">\n");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"page\" ");
  fprintf(cgiOut, "value=\"");
  tempcounter = 0;
  if(pagecounter > pagenumber) tempcounter = pagenumber + 1;
  else tempcounter = pagecounter;
  fprintf(cgiOut, "%d", tempcounter);
  fprintf(cgiOut, "\">\n");
  fprintf(cgiOut, "<th width=5>");
  fprintf(cgiOut, "<input type=\"submit\" value=\"1 >\">");
  fprintf(cgiOut, "</th>");
  fprintf(cgiOut, "</form>");
  // goto last page
  fprintf(cgiOut, "<form action=\"zenginstore.cgi\" method=\"post\">");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"zengincounter\" ");
  fprintf(cgiOut, "value=\"");
  fprintf(cgiOut, "%d", zengincounter);
  fprintf(cgiOut, "\">\n");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"pagecounter\" ");
  fprintf(cgiOut, "value=\"");
  fprintf(cgiOut, "%d", pagecounter);
  fprintf(cgiOut, "\">\n");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"page\" ");
  fprintf(cgiOut, "value=\"");
  fprintf(cgiOut, "%d", pagecounter);
  fprintf(cgiOut, "\">\n");
  fprintf(cgiOut, "<th width=5>");
  fprintf(cgiOut, "<input type=\"submit\" value=\">>\">");
  fprintf(cgiOut, "</th>");
  fprintf(cgiOut, "</form>");

  // filler 3
  fprintf(cgiOut, "<th width=15>");
  fprintf(cgiOut, "&nbsp;");
  fprintf(cgiOut, "</th>\n");

  // goto page number
  fprintf(cgiOut, "<form action=\"zenginstore.cgi\" method=\"post\">");
  fprintf(cgiOut, "<th width=10 nowrap>");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"zengincounter\" ");
  fprintf(cgiOut, "value=\"");
  fprintf(cgiOut, "%d", zengincounter);
  fprintf(cgiOut, "\">\n");
  fprintf(cgiOut, "<input type=\"hidden\" name=\"pagecounter\" ");
  fprintf(cgiOut, "value=\"");
  fprintf(cgiOut, "%d", pagecounter);
  fprintf(cgiOut, "\">\n");
  fprintf(cgiOut, "<input type=\"submit\" value=\"Goto\">");
  fprintf(cgiOut, "</th>");
  fprintf(cgiOut, "<th width=10 nowrap>");
  fprintf(cgiOut, "<input type=\"text\" name=\"page\" size=4 value=");
  fprintf(cgiOut, "%d>\n", pagecounter);
  fprintf(cgiOut, "</th>");
  fprintf(cgiOut, "</tr>\n");
  fprintf(cgiOut, "</form>");
  fprintf(cgiOut, "</table>\n");

/* ---------------------------------------------------------------------------*
 * end the html output                                                        *
 * ---------------------------------------------------------------------------*/

  pagefoot();
  return(0);
}
